package mi.hdm.controllers;

import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import mi.hdm.TastyPages;
import mi.hdm.recipes.RecipeManager;
import mi.hdm.shoppingList.ShoppingList;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.IOException;

public enum View {
    MAIN("/fxml/Tasty_Pages_Main_Page.fxml", "Tasty Pages"),
    RECIPE_VIEW("/fxml/recipe-view.fxml", "Tasty Pages - Recipe View"),
    SHOPPINGLIST_VIEW("/fxml/shoppingList-view.fxml", "Tasty Pages - Shopping List View");

    private static final Logger log = LogManager.getLogger(View.class);

    private final RecipeManager recipeManager = TastyPages.recipeManager;
    private final ShoppingList shoppingList = new ShoppingList();

    private final String path;
    private final String windowTitle;

    View(String path, String windowTitle) {
        this.path = path;
        this.windowTitle = windowTitle;
    }

    public Parent getScene() throws IOException {
        //TODO: keep track of current view and don't reload it if the user selects it again!
        log.info("Loading FXML for '{}' view from: {}", windowTitle, path);
        final FXMLLoader loader = new FXMLLoader(getClass().getResource(path));
        log.debug("Successfully retrieved resource from {}", path);

        //set the correct controller factory for views (necessary to enable dependency injection)
        switch (this) {
            case MAIN -> loader.setControllerFactory((callback) ->
                    new MainPageController(recipeManager)
            );

            case RECIPE_VIEW -> loader.setControllerFactory((callback) ->
                    new RecipeViewController(recipeManager)
            );

            case SHOPPINGLIST_VIEW -> loader.setControllerFactory((callback) ->
                    new ShoppingListViewController(recipeManager, shoppingList)
            );
        }
        log.debug("Set controller factory for {}", windowTitle);

        return loader.load();
    }

    public String getWindowTitle() {
        return windowTitle;
    }
}
